import store from '@/store/index.js'
import router from '@/router/index.js'
import { makeRequest } from '@/libs/RequestModule'
import { setLocalVar } from '@/libs/Util.js'
import { log } from '@/libs/Logger.js'
import md5 from 'md5'
import moment from 'moment'

const doLogOut = function () {
    setLocalVar('SESSIONTOKEN', null)

    const password = store.getters.getLoginFormData[2].value === '%use@stored@pass%' ?
        store.getters.getRefreshToken :
        md5(`odhen${store.getters.getLoginFormData[2].value}7833`)

    store.dispatch('setAllUserData', {
        userName: store.getters.getLoginFormData[1].value,
        refreshToken: password,
        shouldRememberPassword: store.getters.getLoginFormData[3].value
    })
    router.push({ name: 'login' })
}

const _doLogin = async function (payload) {
    const selectedLoginOption = store.getters.getSelectedLoginOption
    if (process.env.usa_instituicao_cliente_filial === 'S' && selectedLoginOption) {
        payload.client = selectedLoginOption.CDCLIENTE
        payload.filial = selectedLoginOption.CDFILIAL
    }

    const response = await makeRequest('login', payload)
    if (!(response?.data?.token === md5(`teknisa ${moment().format('DDMMYYYY')}`))) {
        response.errMessage = response?.data?.message || response?.message
        return response
    }
    setLocalVar('SESSIONTOKEN', response?.data?.ACCESSTOKEN)
    return response
}

const doLogin = async function (refresh = false) {
    const user = store.getters.getLoginFormData[1].value
    const password = store.getters.getLoginFormData[2].value === '%use@stored@pass%' ?
        store.getters.getRefreshToken :
        md5(`odhen${store.getters.getLoginFormData[2].value}7833`)

    const rmbpassword = store.getters.getLoginFormData[3].value

    const atLoginPage = router.currentRoute.value.name === 'login'
    const loginControlsAvailability = store.getters.loginControlsEnabled === true
    const allFieldsFilled = !!user && !!password
    const baseInfo = store.getters.getBaseInfo

    if (atLoginPage && !refresh) {
        if (loginControlsAvailability) {
            if (!allFieldsFilled) {
                await store.dispatch('showModalInfo', { type: 'error', text: 'Todos os campos são obrigatórios', open: true })
                return false
            }
            await store.dispatch('setLoginControlsEnabled', !store.getters.loginControlsEnabled)

            const response = await _doLogin({
                username: user,
                password: password,
                client: baseInfo.CDCLIENTE,
                filial: baseInfo.CDFILIAL
            })

            await store.dispatch('setLoginControlsEnabled', !store.getters.loginControlsEnabled)

            if (response.error) {
                let message = response.errMessage || 'Não foi possível realizar o login'
                if (response.code && response.code === 'ERR_NETWORK') {
                    message = 'Sem conexão. Verifique a conexão com a internet e tente novamente.'
                }
                await store.dispatch('showModalInfo', { type: 'error', text: message, open: true })
                return false
            }

            if (!response.data.showSelectPopup) {
                await store.dispatch('setAllUserData', {
                    userName: user,
                    refreshToken: password,
                    shouldRememberPassword: rmbpassword
                })
            }

            return response
        }
    } else {
        const response = await _doLogin({
            username: store.getters.getUsername,
            password: store.getters.getRefreshToken,
            client: baseInfo.CDCLIENTE,
            filial: baseInfo.CDFILIAL
        })

        if (response.errMessage) {
            const message = 'Não foi possível autenticar'
            await store.dispatch('showModalInfo', { type: 'error', text: message, open: true })
            doLogOut()
        }
        return response
    }
}

const getClients = async function () {
    const response = (await makeRequest('getClients'))?.data
    return response
}

const forgotPassword = async function (CDCLIENTE, NMLOGINCONS) {
    const response = (await makeRequest('forgotPassword', { CDCLIENTE, NMLOGINCONS }))?.data
    return response
}

const updatePasswordAfterRecover = async function (CDCLIENTE, NMLOGINCONS, newPassword) {
    const response = (await makeRequest('updatePasswordAfterRecover', { CDCLIENTE, NMLOGINCONS, newPassword }))?.data
    return response
}

const getRetailGeneralParams = async () => {
    await makeRequest('getRetailGeneralParams', {})
        .then(response => {
            if (response?.sucesso && response?.data) {
                store.dispatch('setRetailGeneralParams', response.data)
                if (response.data.HABCADASTRO) {
                    store.dispatch('setShowRegisterButton', response.data.HABCADASTRO === 'S')
                }
            }
        })
        .catch(error => {
            log({ type: 'error', message: `Erro em getRetailGeneralParams:\n${error.message}` })
        })
}

export {
    doLogin,
    doLogOut,
    getClients,
    forgotPassword,
    updatePasswordAfterRecover,
    getRetailGeneralParams
}
